import { cryptoInternals } from './crypto.js';

function isNonNegativeInteger(n) {
  return typeof n === 'number' && n % 1 === 0 && n >= 0;
}

export function generateIdentityKeyPair() {
  return cryptoInternals.createKeyPair();
}

export function generateRegistrationId() {
  var registrationId = new Uint16Array(cryptoInternals.getRandomBytes(2))[0];
  return registrationId & 0x3fff;
}

export function generateSignedPreKey(identityKeyPair, signedKeyId) {
  if (
    !(identityKeyPair.privKey instanceof ArrayBuffer) ||
    identityKeyPair.privKey.byteLength != 32 ||
    !(identityKeyPair.pubKey instanceof ArrayBuffer) ||
    identityKeyPair.pubKey.byteLength != 33
  ) {
    throw new TypeError('Invalid argument for identityKeyPair');
  }
  if (!isNonNegativeInteger(signedKeyId)) {
    throw new TypeError('Invalid argument for signedKeyId: ' + signedKeyId);
  }

  return cryptoInternals.createKeyPair().then(function (keyPair) {
    return cryptoInternals
      .Ed25519Sign(identityKeyPair.privKey, keyPair.pubKey)
      .then(function (sig) {
        return {
          keyId: signedKeyId,
          keyPair: keyPair,
          signature: sig,
        };
      });
  });
}

export function generatePreKey(keyId) {
  if (!isNonNegativeInteger(keyId)) {
    throw new TypeError('Invalid argument for keyId: ' + keyId);
  }

  return cryptoInternals.createKeyPair().then(function (keyPair) {
    return { keyId: keyId, keyPair: keyPair };
  });
}
