import curve from './curve.js';

class WorkerClient {
  constructor() {
    this.jobs = {};
    this.jobId = 0;
    this.worker = new Worker('/base/dist/libsignal-worker.js');
    this.worker.onmessage = e => {
      console.debug(e.data);
      var job = this.jobs[e.data.id];
      if (e.data.error && typeof job.onerror === 'function') {
        job.onerror(new Error(e.data.error));
      } else if (typeof job.onsuccess === 'function') {
        job.onsuccess(e.data.result);
      }
      delete this.jobs[e.data.id];
    };
  }

  stop() {
    this.worker.terminate();
  }

  postMessage(methodName, args, _onsuccess, _onerror) {
    console.debug(methodName);
    return new Promise((resolve, reject) => {
      this.jobs[this.jobId] = { onsuccess: resolve, onerror: reject };
      this.worker.postMessage({
        id: this.jobId,
        methodName: methodName,
        args: args,
      });
      this.jobId++;
    });
  }

  keyPair(privKey) {
    return this.postMessage('keyPair', [privKey]);
  }
  sharedSecret(pubKey, privKey) {
    return this.postMessage('sharedSecret', [pubKey, privKey]);
  }
  sign(privKey, message) {
    return this.postMessage('sign', [privKey, message]);
  }
  verify(pubKey, message, sig) {
    return this.postMessage('verify', [pubKey, message, sig]);
  }
}

export function startWorker() {
  stopWorker();
  curve.worker = new WorkerClient();
}

export function stopWorker() {
  const worker = curve.worker;
  if (worker) {
    delete curve.worker;
    return worker.stop();
  }
}
