import { cryptoInternals } from '../../src/crypto.js';
import SignalProtocolAddress from '../../src/signal_protocol_address.js';
import { assertEqualArrayBuffers } from '../helpers.js';

export default function testPreKeyStore(store) {
  var number = '+5558675309';
  var testKey;
  describe('PreKeyStore', function () {
    before(function () {
      return cryptoInternals.createKeyPair().then(function (keyPair) {
        testKey = keyPair;
      });
    });
    describe('storePreKey', function () {
      it('stores prekeys', function () {
        var address = new SignalProtocolAddress(number, 1);
        return store.storePreKey(address.toString(), testKey).then(function () {
          return store.loadPreKey(address.toString()).then(function (key) {
            assertEqualArrayBuffers(key.pubKey, testKey.pubKey);
            assertEqualArrayBuffers(key.privKey, testKey.privKey);
          });
        });
      });
    });
    describe('loadPreKey', function () {
      it('returns prekeys that exist', function () {
        var address = new SignalProtocolAddress(number, 1);
        return store.storePreKey(address.toString(), testKey).then(function () {
          return store.loadPreKey(address.toString()).then(function (key) {
            assertEqualArrayBuffers(key.pubKey, testKey.pubKey);
            assertEqualArrayBuffers(key.privKey, testKey.privKey);
          });
        });
      });
      it('returns undefined for prekeys that do not exist', function () {
        new SignalProtocolAddress(number, 2);
        return store.loadPreKey(2).then(function (key) {
          assert.isUndefined(key);
        });
      });
    });
    describe('removePreKey', function () {
      it('deletes prekeys', function () {
        var address = new SignalProtocolAddress(number, 2);
        before(function () {
          return store.storePreKey(address.toString(), testKey);
        });
        return store.removePreKey(address.toString()).then(function () {
          return store.loadPreKey(address.toString()).then(function (key) {
            assert.isUndefined(key);
          });
        });
      });
    });
  });
}
