import { cryptoInternals } from '../../src/crypto.js';
import { assertEqualArrayBuffers } from '../helpers.js';

export default function testSignedPreKeyStore(store) {
  describe('SignedPreKeyStore', function () {
    var testKey;
    before(function () {
      return cryptoInternals.createKeyPair().then(function (keyPair) {
        testKey = keyPair;
      });
    });
    describe('storeSignedPreKey', function () {
      it('stores signed prekeys', function () {
        return store.storeSignedPreKey(3, testKey).then(function () {
          return store.loadSignedPreKey(3).then(function (key) {
            assertEqualArrayBuffers(key.pubKey, testKey.pubKey);
            assertEqualArrayBuffers(key.privKey, testKey.privKey);
          });
        });
      });
    });
    describe('loadSignedPreKey', function () {
      it('returns prekeys that exist', function () {
        return store.storeSignedPreKey(1, testKey).then(function () {
          return store.loadSignedPreKey(1).then(function (key) {
            assertEqualArrayBuffers(key.pubKey, testKey.pubKey);
            assertEqualArrayBuffers(key.privKey, testKey.privKey);
          });
        });
      });
      it('returns undefined for prekeys that do not exist', function () {
        return store.storeSignedPreKey(1, testKey).then(function () {
          return store.loadSignedPreKey(2).then(function (key) {
            assert.isUndefined(key);
          });
        });
      });
    });
    describe('removeSignedPreKey', function () {
      it('deletes signed prekeys', function () {
        before(function () {
          return store.storeSignedPreKey(4, testKey);
        });
        return store.removeSignedPreKey(4, testKey).then(function () {
          return store.loadSignedPreKey(4).then(function (key) {
            assert.isUndefined(key);
          });
        });
      });
    });
  });
}
