import * as path from 'path';

import { CleanWebpackPlugin } from 'clean-webpack-plugin';
import CopyPlugin from 'copy-webpack-plugin';
import * as webpack from 'webpack';
import { WebpackManifestPlugin } from 'webpack-manifest-plugin';

const { ProvidePlugin } = webpack;

const config: webpack.Configuration = {
  mode: 'development',
  entry: './src/index.js',
  // 'libsignal-worker': './src/worker.js',
  // test: './test/index.js',
  output: {
    filename: '[name].js',
    path: path.resolve(__dirname, 'dist'),
    library: 'libsignal',
  },
  devtool: 'source-map',
  plugins: [
    new CleanWebpackPlugin(),
    new ProvidePlugin({
      assert: ['chai', 'assert'],
      expect: ['chai', 'expect'],
    }),
    new WebpackManifestPlugin(),
  ],
  resolve: {
    fallback: {
      fs: false,
      path: false,
    },
    alias: {
      bytebuffer: 'bytebuffer/dist/bytebuffer-dataview',
    },
  },
  module: {
    rules: [
      {
        test: path.resolve(__dirname, 'src/curve25519.js'),
        loader: 'string-replace-loader',
        options: {
          search: /var wasmBinaryFile = '.*';/,
          replace:
            "import wasmAsset from './curve25519.wasm';\nvar wasmBinaryFile = wasmAsset.split('/').pop();",
        },
      },
      {
        test: /\.wasm$/,
        loader: 'file-loader',
        options: {
          name: '[name]-[contenthash].[ext]',
        },
      },
    ],
  },
  optimization: {
    splitChunks: {
      chunks: 'async',
    },
  },
};

export default config;
